"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

__all__ = ["create_deployment_from_git_sync"]

from cfnlint.runner.deployment_file.deployment_types.git_sync import (
    create_deployment_from_git_sync,
)
